
import sys, linecache



# Function to get exception from stack
def _get_exception(debug:bool = True):
    # print 'In # print exception'
    exc_type, exc_obj, tb = sys.exc_info()
    f = tb.tb_frame
    lineno = tb.tb_lineno
    filename = f.f_code.co_filename
    linecache.checkcache(filename)
    line = linecache.getline(filename, lineno, f.f_globals)
    err_obj = {
        'error_code':str(exc_type),
        'error_message':str(exc_obj),
        }
    if debug:
        err_obj['error_at']='EXCEPTION IN ({}, LINE {} "{}")'.format(filename, lineno, line.strip())
    return err_obj

#Function to create user defined exception object
def _errorify(error_code='', error_msg='', error_at=''):
    error_obj = {
        'error_code':error_code,
        'error_message':error_msg,
        'error_at':error_at
    }
    return error_obj



# get nse daily bhav
# https://www1.nseindia.com/content/historical/EQUITIES/2020/JUN/cm12JUN2020bhav.csv.zip
from datetime import datetime, timedelta
from typing import Optional

import requests
import os
from pathlib import Path
from requests.adapters import HTTPAdapter
from requests.packages.urllib3 import Retry

from .common import _get_exception

from multiprocessing.pool import ThreadPool

class NSEDaily(object):
    def __init__(self,
                 default_date_pattern: Optional[str] = '%Y%m%d',
                 nse_file_pattern: Optional[str] = "cm{date_part}bhav.csv.zip",
                 nse_file_date_part_format: Optional[str] = '%d%b%Y',
                 nse_uri_pattern: Optional[
                     str] = "https://www1.nseindia.com/content/historical/EQUITIES/{yyyy_mon}/{file_name}",
                 nse_uri_yy_mm_format: Optional[str] = '%Y/%b',
                 download_path: Optional[str] = None
                 ):
        self.default_date_pattern = default_date_pattern
        self.nse_file_pattern = nse_file_pattern
        self.nse_file_date_part_format = nse_file_date_part_format
        self.nse_uri_pattern = nse_uri_pattern
        self.nse_uri_yy_mm_format = nse_uri_yy_mm_format
        self.download_path = download_path
        if self.download_path is None or str(self.download_path).strip() == '':
            appdir =str(Path.cwd())
            self.download_path = os.path.join(appdir, 'downloads')
        self._create_session()

    def _create_session(self):
        self.session = requests.Session()
        retry = Retry(total=5,
                      read=5,
                      connect=5,
                      status_forcelist=(500,502,504),
                      method_whitelist=frozenset(['GET','POST']),
                      backoff_factor=1
                      )
        adapter = HTTPAdapter(max_retries=retry)
        self.session.mount('http://',adapter)
        self.session.mount('https://',adapter)
    def _check_reponse(self,response:requests.Response):
        try:
            response.raise_for_status()
            return True
        except requests.exceptions.HTTPError:
            print('HTTP Error %s',response.reason)
            print(response.text)
            return False

    def _download_by_date(self,file_date:datetime):
        daynum = file_date.weekday()
        if daynum>=5:
            print("{} is weekend, file skipped".format(file_date.strftime(self.default_date_pattern)))
            return file_date,None
        file_date_str = file_date.strftime(self.nse_file_date_part_format).upper()
        nse_yy_mm = file_date.strftime(self.nse_uri_yy_mm_format).upper()
        nse_file_name = self.nse_file_pattern.format(date_part=file_date_str)
        nse_uri = self.nse_uri_pattern.format(yyyy_mon=nse_yy_mm, file_name=nse_file_name)
        nse_download_file_path = os.path.join(self.download_path, nse_file_name)
        if not os.path.exists(self.download_path):
            os.makedirs(self.download_path)
        response = self.session.request(method='GET',url=nse_uri,allow_redirects=True)
        # r = requests.get(nse_uri, allow_redirects=True)
        status = self._check_reponse(response)
        if not status:
            return file_date,None
        content_type = response.headers.get('content-type')
        if content_type == 'application/zip':
            with open(nse_download_file_path, 'wb') as nse_file:
                nse_file.write(response.content)
        print("{} download complete".format(nse_file_name))
        return file_date,nse_download_file_path

    def download_by_date(self, date_str,date_format:Optional[str]='%Y%m%d'):
        try:
            file_date = datetime.strptime(date_str,date_format)
            return self._download_by_date(file_date)
        except:
            e = _get_exception()
            raise Exception(e)

    def download_by_date_range(self, date_start:str,date_end:str,date_format:Optional[str]='%Y%m%d',num_workers:Optional[int]=1):
        try:
            start = datetime.strptime(date_start,date_format)
            end = datetime.strptime(date_end,date_format)
            l_dates = [start+timedelta(days=x) for x in range(0,(end-start).days)]
            tpool = ThreadPool(processes=num_workers)
            l_e = tpool.map(self._download_by_date,l_dates)
            return l_e
        except:
            e = _get_exception()
            raise Exception(e)


from setuptools import setup, find_packages
print("Packages found:",find_packages())
# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name = "get_nse_daily",
    version="0.2",
    packages=find_packages(),
    author="Tanveer Hora",
    author_email = 'tanveer.hora@gmail.com',
    license='MIT',
    description="pure python library built using requests to get daily day end data from nse india",
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords = "nse,bhavcopy,bhav",
    install_requires = [
        'requests>=2.26.0'
    ],
    url = "https://github.com/v33rh0ra/get_nse_daily",#project home page from github
    download_url = 'https://github.com/v33rh0ra/get_nse_daily/archive/v_02.tar.gz',
    project_urls={
        "Documentation":"https://v33rh0ra.github.io/get_nse_daily/nse_daily/index.html",
        "Bug Tracker":"https://github.com/v33rh0ra/get_nse_daily/issues",
        "Source Code":"https://github.com/v33rh0ra/get_nse_daily",
    },
    classifiers=[
        "Operating System :: OS Independent",
        'Programming Language :: Python :: 3',
        "Programming Language :: Python :: 3.7",
        "Topic :: Scientific/Engineering",
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        'Intended Audience :: Developers',
    ],
)